@@ -1,3 +1,4 @@
+#![cfg_attr(not(feature = "macros"), allow(rustdoc::broken_intra_doc_links))]
 /*!
 # Strongly-typed strings for use with XML 1.0 documents
 
@@ -13,7 +14,7 @@ times.
 
 - [`Name`] and [`NameStr`] represent the `Name` production and can be used
   for element and attribute names before namespace prefix expansion.
-- [`NCName`] and [`NCNameStr`] represent the `Name` production but without a
+- [`NcName`] and [`NcNameStr`] represent the `Name` production but without a
   colon inside; they are used for localnames after prefix expansion and to
   carry the prefixes themselves.
 - [`CData`] and [`CDataStr`] represent strings of XML `Char`s, which are
@@ -25,24 +26,22 @@ times.
 
 ## Construction
 
-To construct [`str`]-like references at compile time, you can use the macros
-from the [`rxml_proc`](https://docs.rs/rxml_proc) crate. It offers
-`xml_cdata!`, `xml_name!` and `xml_ncname!` macros which convert a string
-literal into `&CDataStr`, `&NameStr` and `&NCNameStr` (respectively) with
-validation at compile time.
+To construct [`str`]-like references from a literal, you can use the macros
+offered when this crate is built with the `macros` feature: [`xml_name!`],
+[`xml_ncname!`], [`xml_cdata!`].
 
 In general, owned values are constructed using the [`std::convert::TryInto`]
 mechanism, from other string types. Supported source types are:
 
-* [`String`] (copies for [`Name`] and [`NCName`], moves for [`CData`])
-* [`smartstring::alias::String`] (copies for [`CData`], moves for [`Name`] and [`NCName`])
+* [`String`] (copies for [`Name`] and [`NcName`], moves for [`CData`])
+* [`smartstring::alias::String`] (copies for [`CData`], moves for [`Name`] and [`NcName`])
 * [`str`] (copies for all types except the slice types)
 
 In addition, the following conversions can be done without extra checking and
 are possible through `.into()`:
 
-* [`NCName`] to [`Name`]
-* [`NCName`] to [`CData`]
+* [`NcName`] to [`Name`]
+* [`NcName`] to [`CData`]
 * [`Name`] to [`CData`]
 
 (and likewise for the corresponding Str types)
@@ -50,17 +49,25 @@ are possible through `.into()`:
 The inverse directions are only available through `try_into`.
 */
 
-use crate::error::{NWFError, WFError, ERRCTX_UNKNOWN};
-use rxml_validation::selectors;
-use rxml_validation::selectors::CharSelector;
-pub use rxml_validation::{validate_cdata, validate_name, validate_ncname};
-use smartstring::alias::String as SmartString;
 use std::borrow::{Borrow, Cow, ToOwned};
 use std::cmp::{Ordering, PartialOrd};
 use std::convert::{TryFrom, TryInto};
 use std::fmt;
 use std::ops::{Add, Deref};
 
+use smartstring::alias::String as SmartString;
+
+use rxml_validation::selectors;
+use rxml_validation::selectors::CharSelector;
+use rxml_validation::{
+	validate_cdata as raw_validate_cdata, validate_name as raw_validate_name,
+	validate_ncname as raw_validate_ncname, Error as ValidationError,
+};
+
+use crate::error::{XmlError, ERRCTX_UNKNOWN};
+
+use super::errctx;
+
 macro_rules! rxml_unsafe_str_construct_doc {
 	($name:ident, $other:ident) => {
 		concat!(
@@ -111,19 +118,19 @@ macro_rules! rxml_custom_string_type {
 		impl $name {
 			#[deprecated(since = "0.4.0", note = "use the TryFrom<> trait implementation instead")]
 			#[doc = rxml_safe_str_construct_doc!($name, str, "")]
-			pub fn from_str<T: AsRef<str>>(s: T) -> Result<Self, WFError> {
+			pub fn from_str<T: AsRef<str>>(s: T) -> Result<Self, XmlError> {
 				s.as_ref().try_into()
 			}
 
 			#[deprecated(since = "0.4.0", note = "use the TryFrom<> trait implementation instead")]
 			#[doc = rxml_safe_str_construct_doc!($name, String, "")]
-			pub fn from_string<T: Into<String>>(s: T) -> Result<Self, WFError> {
+			pub fn from_string<T: Into<String>>(s: T) -> Result<Self, XmlError> {
 				s.into().try_into()
 			}
 
 			#[deprecated(since = "0.4.0", note = "use the TryFrom<> trait implementation instead")]
 			#[doc = rxml_safe_str_construct_doc!($name, SmartString, "")]
-			pub fn from_smartstring<T: Into<SmartString>>(s: T) -> Result<Self, WFError> {
+			pub fn from_smartstring<T: Into<SmartString>>(s: T) -> Result<Self, XmlError> {
 				s.into().try_into()
 			}
 
@@ -288,7 +295,7 @@ macro_rules! rxml_custom_string_type {
 		}
 
 		impl TryFrom<SmartString> for $name {
-			type Error = WFError;
+			type Error = XmlError;
 
 			#[doc = rxml_safe_str_construct_doc!($name, SmartString, "")]
 			fn try_from(other: SmartString) -> Result<Self, Self::Error> {
@@ -298,7 +305,7 @@ macro_rules! rxml_custom_string_type {
 		}
 
 		impl TryFrom<String> for $name {
-			type Error = WFError;
+			type Error = XmlError;
 
 			#[doc = rxml_safe_str_construct_doc!($name, String, "")]
 			fn try_from(other: String) -> Result<Self, Self::Error> {
@@ -308,7 +315,7 @@ macro_rules! rxml_custom_string_type {
 		}
 
 		impl TryFrom<&str> for $name {
-			type Error = WFError;
+			type Error = XmlError;
 
 			#[doc = rxml_safe_str_construct_doc!($name, str, "")]
 			fn try_from(other: &str) -> Result<Self, Self::Error> {
@@ -327,7 +334,7 @@ macro_rules! rxml_custom_string_type {
 			type Output = $name;
 
 			fn add(self, rhs: &$borrowed) -> Self::Output {
-				// SAFETY: for Name, NCName and CData, a concatenation with
+				// SAFETY: for Name, NcName and CData, a concatenation with
 				// strings of the same type is always also of the same type.
 				// (NB: A subslice might not be, because e.g. Name has
 				// constraints about what might occur in the first codepoint).
@@ -349,7 +356,7 @@ macro_rules! rxml_custom_str_type {
 
 		impl $name {
 			#[doc = rxml_safe_str_construct_doc!($name, str, "")]
-			pub fn from_str<'x>(s: &'x str) -> Result<&'x Self, WFError> {
+			pub fn from_str<'x>(s: &'x str) -> Result<&'x Self, XmlError> {
 				s.try_into()
 			}
 
@@ -414,7 +421,7 @@ macro_rules! rxml_custom_str_type {
 		}
 
 		impl<'x> TryFrom<&'x str> for &'x $name {
-			type Error = WFError;
+			type Error = XmlError;
 
 			fn try_from(other: &'x str) -> Result<Self, Self::Error> {
 				$check(other)?;
@@ -480,7 +487,7 @@ rxml_custom_string_type_pair! {
 	///                        | [#x0300-#x036F] | [#x203F-#x2040]
 	/// [5]  Name          ::= NameStartChar (NameChar)*
 	/// ```
-	pub struct Name(SmartString) use validate_name;
+	pub struct Name(SmartString) use raw_validate_name;
 
 	/// str which conforms to the Name production of XML 1.0.
 	///
@@ -506,14 +513,18 @@ impl Name {
 	///
 	/// If neither of the two cases apply or the string on either side of the
 	/// colon is empty, an error is returned.
-	pub fn split_name(self) -> Result<(Option<NCName>, NCName), NWFError> {
+	///
+	/// This function optimizes the split (compared to operating on a borrowed
+	/// [`NameStr`] and then cloning the returned parts) by avoiding
+	/// unnecessary copying.
+	pub fn split_name(self) -> Result<(Option<NcName>, NcName), XmlError> {
 		let mut name = self.0;
 		let colon_pos = match name.find(':') {
-			None => return Ok((None, unsafe { NCName::from_smartstring_unchecked(name) })),
+			None => return Ok((None, unsafe { NcName::from_smartstring_unchecked(name) })),
 			Some(pos) => pos,
 		};
 		if colon_pos == 0 || colon_pos == name.len() - 1 {
-			return Err(NWFError::EmptyNamePart(ERRCTX_UNKNOWN));
+			return Err(XmlError::EmptyNamePart(ERRCTX_UNKNOWN));
 		}
 
 		let localname = name.split_off(colon_pos + 1);
@@ -521,11 +532,11 @@ impl Name {
 
 		if localname.find(':').is_some() {
 			// Namespaces in XML 1.0 (Third Edition) namespace-well-formed criterium 1
-			return Err(NWFError::MultiColonName(ERRCTX_UNKNOWN));
+			return Err(XmlError::MultiColonName(ERRCTX_UNKNOWN));
 		};
 		if !selectors::CLASS_XML_NAMESTART.select(localname.chars().next().unwrap()) {
-			// Namespaces in XML 1.0 (Third Edition) NCName production
-			return Err(NWFError::InvalidLocalName(ERRCTX_UNKNOWN));
+			// Namespaces in XML 1.0 (Third Edition) NcName production
+			return Err(XmlError::InvalidLocalName(ERRCTX_UNKNOWN));
 		}
 
 		prefix.pop();
@@ -536,71 +547,114 @@ impl Name {
 		debug_assert!(prefix.len() > 0);
 		debug_assert!(localname.len() > 0);
 		Ok((
-			Some(unsafe { NCName::from_smartstring_unchecked(prefix) }),
-			unsafe { NCName::from_smartstring_unchecked(localname) },
+			Some(unsafe { NcName::from_smartstring_unchecked(prefix) }),
+			unsafe { NcName::from_smartstring_unchecked(localname) },
 		))
 	}
 }
 
 impl NameStr {
+	/// Create an owned copy of the string as [`Name`].
+	///
+	/// This operation is also available as implementation of the `Into`
+	/// trait.
 	pub fn to_name(&self) -> Name {
 		self.into()
 	}
-}
 
-impl From<NCName> for Name {
-	fn from(other: NCName) -> Self {
-		other.as_name()
+	/// Split the name at a colon, if it exists.
+	///
+	/// If the name contains no colon, the function returns `(None, self)`.
+	/// If the name contains exactly one colon, the function returns the part
+	/// before the colon (the prefix) in the first return value and the part
+	/// following the colon (the suffix) as second return value.
+	///
+	/// If neither of the two cases apply or the string on either side of the
+	/// colon is empty, an error is returned.
+	pub fn split_name(&self) -> Result<(Option<&'_ NcNameStr>, &'_ NcNameStr), XmlError> {
+		let name = &self.0;
+		let colon_pos = match name.find(':') {
+			None => return Ok((None, unsafe { NcNameStr::from_str_unchecked(name) })),
+			Some(pos) => pos,
+		};
+		if colon_pos == 0 || colon_pos == name.len() - 1 {
+			return Err(XmlError::EmptyNamePart(ERRCTX_UNKNOWN));
+		}
+
+		let (prefix, localname) = name.split_at(colon_pos);
+		let localname = &localname[1..];
+
+		if localname.find(':').is_some() {
+			// Namespaces in XML 1.0 (Third Edition) namespace-well-formed criterium 1
+			return Err(XmlError::MultiColonName(ERRCTX_UNKNOWN));
+		};
+		if !selectors::CLASS_XML_NAMESTART.select(localname.chars().next().unwrap()) {
+			// Namespaces in XML 1.0 (Third Edition) NcName production
+			return Err(XmlError::InvalidLocalName(ERRCTX_UNKNOWN));
+		}
+
+		debug_assert!(prefix.len() > 0);
+		debug_assert!(localname.len() > 0);
+		Ok((
+			Some(unsafe { NcNameStr::from_str_unchecked(prefix) }),
+			unsafe { NcNameStr::from_str_unchecked(localname) },
+		))
 	}
 }
 
-impl<'x> From<&'x NCNameStr> for &'x NameStr {
-	fn from(other: &'x NCNameStr) -> Self {
+impl From<NcName> for Name {
+	fn from(other: NcName) -> Self {
+		other.into_name()
+	}
+}
+
+impl<'x> From<&'x NcNameStr> for &'x NameStr {
+	fn from(other: &'x NcNameStr) -> Self {
 		other.as_namestr()
 	}
 }
 
 rxml_custom_string_type_pair! {
-	/// String which conforms to the NCName production of Namespaces in XML 1.0.
+	/// String which conforms to the NcName production of Namespaces in XML 1.0.
 	///
-	/// [`NCName`] corresponds to a (restricted) [`String`]. For a [`str`]-like
-	/// type with the same restrictions, see [`NCNameStr`]. `&NCNameStr` can be
+	/// [`NcName`] corresponds to a (restricted) [`String`]. For a [`str`]-like
+	/// type with the same restrictions, see [`NcNameStr`]. `&NcNameStr` can be
 	/// created from a string literal at compile time using the `xml_ncname` macro
 	/// from [`rxml_proc`](https://docs.rs/rxml_proc).
 	///
-	/// Since [`NCName`] (indirectly) derefs to [`str`], all (non-mutable)
+	/// Since [`NcName`] (indirectly) derefs to [`str`], all (non-mutable)
 	/// methods from [`str`] are available.
 	///
 	/// # Formal definition
 	///
-	/// The data inside [`NCName`] (and [`NCNameStr`]) is guaranteed to conform to
-	/// the `NCName` production of the below grammar, quoted from
-	/// [Namespaces in XML 1.0 § 3](https://www.w3.org/TR/REC-xml-names/#NT-NCName):
+	/// The data inside [`NcName`] (and [`NcNameStr`]) is guaranteed to conform to
+	/// the `NcName` production of the below grammar, quoted from
+	/// [Namespaces in XML 1.0 § 3](https://www.w3.org/TR/REC-xml-names/#NT-NcName):
 	///
 	/// ```text
-	/// [4] NCName ::= Name - (Char* ':' Char*)  /* An XML Name, minus the ":" */
+	/// [4] NcName ::= Name - (Char* ':' Char*)  /* An XML Name, minus the ":" */
 	/// ```
-	pub struct NCName(SmartString) use validate_ncname;
+	pub struct NcName(SmartString) use raw_validate_ncname;
 
-	/// str which conforms to the NCName production of Namespaces in XML 1.0.
+	/// str which conforms to the NcName production of Namespaces in XML 1.0.
 	///
-	/// [`NCNameStr`] corresponds to a (restricted) [`str`]. For a [`String`]-like
+	/// [`NcNameStr`] corresponds to a (restricted) [`str`]. For a [`String`]-like
 	/// type with the same restrictions as well as the formal definition of those
-	/// restrictions, see [`NCName`].
+	/// restrictions, see [`NcName`].
 	///
-	/// `&NCNameStr` can be created from a string literal at compile time using
+	/// `&NcNameStr` can be created from a string literal at compile time using
 	/// the `xml_ncname` macro from [`rxml_proc`](https://docs.rs/rxml_proc).
 	///
-	/// Since [`NCNameStr`] derefs to [`str`], all (non-mutable) methods from
+	/// Since [`NcNameStr`] derefs to [`str`], all (non-mutable) methods from
 	/// [`str`] are available.
-	pub struct NCNameStr(str);
+	pub struct NcNameStr(str);
 }
 
-impl NCName {
-	/// Compose two [`NCName`] objects to one [`Name`], separating them with
+impl NcName {
+	/// Compose two [`NcName`] objects to one [`Name`], separating them with
 	/// a colon.
 	///
-	/// As an [`NCName`] is always a valid [`Name`], the composition of the
+	/// As an [`NcName`] is always a valid [`Name`], the composition of the
 	/// two with a `:` as separator is also a valid [`Name`].
 	///
 	/// This is the inverse of [`Name::split_name()`].
@@ -608,45 +662,59 @@ impl NCName {
 	/// # Example
 	///
 	/// ```
-	/// # use rxml::NCName;
-	/// let prefix = NCName::from_str("xmlns").unwrap();
-	/// let localname = NCName::from_str("stream").unwrap();
+	/// # use rxml::NcName;
+	/// let prefix = NcName::from_str("xmlns").unwrap();
+	/// let localname = NcName::from_str("stream").unwrap();
 	/// assert_eq!(prefix.add_suffix(&localname), "xmlns:stream");
 	/// ```
-	pub fn add_suffix(self, suffix: &NCNameStr) -> Name {
+	pub fn add_suffix(self, suffix: &NcNameStr) -> Name {
 		let mut s: String = self.0.into();
 		s.reserve(suffix.len() + 1);
 		s.push_str(":");
 		s.push_str(suffix);
-		// SAFETY: NCName cannot contain a colon; Name is NCName with colons,
-		// so we can concat two NCNames to a Name.
+		// SAFETY: NcName cannot contain a colon; Name is NcName with colons,
+		// so we can concat two NcNames to a Name.
 		unsafe { Name::from_string_unchecked(s) }
 	}
 
-	pub fn as_name(self) -> Name {
-		// SAFETY: NCName is a strict subset of Name
+	/// Convert the [`NcName`] into a [`Name`].
+	///
+	/// This operation is O(1).
+	///
+	/// This operation is also available as implementation of the `Into`
+	/// trait.
+	pub fn into_name(self) -> Name {
+		// SAFETY: NcName is a strict subset of Name
 		unsafe { Name::from_smartstring_unchecked(self.0) }
 	}
 }
 
-impl NCNameStr {
-	pub fn to_ncname(&self) -> NCName {
+impl NcNameStr {
+	/// Create an owned copy of the string as [`NcName`].
+	///
+	/// This operation is also available as implementation of the `Into`
+	/// trait.
+	pub fn to_ncname(&self) -> NcName {
 		self.into()
 	}
 
+	/// Create an owned copy of the string as [`Name`].
 	pub fn to_name(&self) -> Name {
-		self.to_ncname().as_name()
+		self.to_ncname().into()
 	}
 
+	/// Access the string as [`NameStr`].
+	///
+	/// This operation is O(1), as Names are a strict superset of NcNames.
 	pub fn as_namestr<'x>(&'x self) -> &'x NameStr {
-		// SAFETY: NCName is a strict subset of Name
+		// SAFETY: NcName is a strict subset of Name
 		unsafe { NameStr::from_str_unchecked(&self.0) }
 	}
 
-	/// Compose two [`NCName`] objects to one [`Name`], separating them with
+	/// Compose two [`NcName`] objects to one [`Name`], separating them with
 	/// a colon.
 	///
-	/// As an [`NCName`] is always a valid [`Name`], the composition of the
+	/// As an [`NcName`] is always a valid [`Name`], the composition of the
 	/// two with a `:` as separator is also a valid [`Name`].
 	///
 	/// This is the inverse of [`Name::split_name()`].
@@ -654,18 +722,18 @@ impl NCNameStr {
 	/// # Example
 	///
 	/// ```
-	/// # use rxml::NCName;
-	/// let prefix = NCName::from_str("xmlns").unwrap();
-	/// let localname = NCName::from_str("stream").unwrap();
+	/// # use rxml::NcName;
+	/// let prefix = NcName::from_str("xmlns").unwrap();
+	/// let localname = NcName::from_str("stream").unwrap();
 	/// assert_eq!(prefix.add_suffix(&localname), "xmlns:stream");
 	/// ```
-	pub fn with_suffix(&self, suffix: &NCNameStr) -> Name {
+	pub fn with_suffix(&self, suffix: &NcNameStr) -> Name {
 		let mut s = String::with_capacity(self.len() + 1 + suffix.len());
 		s.push_str(self);
 		s.push_str(":");
 		s.push_str(suffix);
-		// SAFETY: NCName cannot contain a colon; Name is NCName with colons,
-		// so we can concat two NCNames to a Name.
+		// SAFETY: NcName cannot contain a colon; Name is NcName with colons,
+		// so we can concat two NcNames to a Name.
 		unsafe { Name::from_string_unchecked(s) }
 	}
 }
@@ -704,7 +772,7 @@ rxml_custom_string_type_pair! {
 	/// expanded by the lexer. This implies that `CData` objects are not safe to
 	/// just verbatimly copy into an XML document; additional escaping may be
 	/// necessary.
-	pub struct CData(String) use validate_cdata;
+	pub struct CData(String) use raw_validate_cdata;
 
 	/// str which consists only of XML 1.0 Chars.
 	///
@@ -721,14 +789,18 @@ rxml_custom_string_type_pair! {
 }
 
 impl CDataStr {
+	/// Create an owned copy of the string as [`CData`].
+	///
+	/// This operation is also available as implementation of the `Into`
+	/// trait.
 	pub fn to_cdata(&self) -> CData {
 		self.into()
 	}
 }
 
-impl From<NCName> for CData {
-	fn from(other: NCName) -> Self {
-		// SAFETY: NCNames can only consist of valid XML 1.0 chars, so they
+impl From<NcName> for CData {
+	fn from(other: NcName) -> Self {
+		// SAFETY: NcNames can only consist of valid XML 1.0 chars, so they
 		// are also valid CData
 		unsafe { CData::from_smartstring_unchecked(other.0) }
 	}
@@ -742,9 +814,9 @@ impl From<Name> for CData {
 	}
 }
 
-impl<'x> From<&'x NCNameStr> for &'x CDataStr {
-	fn from(other: &'x NCNameStr) -> Self {
-		// SAFETY: NCNames can only consist of valid XML 1.0 chars, so they
+impl<'x> From<&'x NcNameStr> for &'x CDataStr {
+	fn from(other: &'x NcNameStr) -> Self {
+		// SAFETY: NcNames can only consist of valid XML 1.0 chars, so they
 		// are also valid CData
 		unsafe { CDataStr::from_str_unchecked(&other.0) }
 	}
@@ -758,17 +830,133 @@ impl<'x> From<&'x NameStr> for &'x CDataStr {
 	}
 }
 
+/**
+Check whether a str is valid XML 1.0 CData
+
+# Example
+
+```rust
+use rxml::error::XmlError;
+use rxml::strings::validate_cdata;
+
+assert!(validate_cdata("foo bar baz <fnord!>").is_ok());
+assert!(matches!(validate_cdata("\x01"), Err(XmlError::UnexpectedChar(_, '\x01', _))));
+*/
+pub fn validate_cdata(s: &str) -> Result<(), XmlError> {
+	match raw_validate_cdata(s) {
+		Ok(()) => Ok(()),
+		Err(ValidationError::InvalidChar(ch)) => {
+			Err(XmlError::UnexpectedChar(errctx::ERRCTX_NAME, ch, None).into())
+		}
+		Err(ValidationError::EmptyName) => unreachable!(),
+	}
+}
+
+/**
+Check whether a str is a valid XML 1.0 Name
+
+**Note:** This does *not* enforce that the name contains only a single colon.
+
+# Example
+
+```rust
+use rxml::error::XmlError;
+use rxml::strings::validate_name;
+
+assert!(validate_name("foobar").is_ok());
+assert!(validate_name("foo:bar").is_ok());
+assert!(matches!(validate_name("foo bar"), Err(XmlError::UnexpectedChar(_, ' ', _))));
+assert!(matches!(validate_name(""), Err(XmlError::InvalidSyntax(_))));
+*/
+pub fn validate_name(s: &str) -> Result<(), XmlError> {
+	match raw_validate_name(s) {
+		Ok(()) => Ok(()),
+		Err(ValidationError::InvalidChar(ch)) => {
+			Err(XmlError::UnexpectedChar(errctx::ERRCTX_NAME, ch, None).into())
+		}
+		Err(ValidationError::EmptyName) => Err(XmlError::InvalidSyntax(errctx::ERRCTX_NAME).into()),
+	}
+}
+
+/**
+Check whether a str is a valid XML 1.0 Name, without colons.
+
+# Example
+
+```rust
+use rxml::error::XmlError;
+use rxml::strings::validate_ncname;
+
+assert!(validate_ncname("foobar").is_ok());
+assert!(matches!(validate_ncname("foo:bar"), Err(XmlError::MultiColonName(_))));
+assert!(matches!(validate_ncname(""), Err(XmlError::EmptyNamePart(_))));
+*/
+pub fn validate_ncname(s: &str) -> Result<(), XmlError> {
+	match raw_validate_ncname(s) {
+		Ok(()) => Ok(()),
+		Err(ValidationError::InvalidChar(':')) => {
+			Err(XmlError::MultiColonName(errctx::ERRCTX_NAME))
+		}
+		Err(ValidationError::InvalidChar(ch)) => {
+			Err(XmlError::UnexpectedChar(errctx::ERRCTX_NAME, ch, None))
+		}
+		Err(ValidationError::EmptyName) => Err(XmlError::EmptyNamePart(errctx::ERRCTX_NAME)),
+	}
+}
+
 #[cfg(test)]
 mod tests {
 	use super::*;
 
 	#[test]
-	fn split_name_rejects_localname_with_non_namestart_first_char() {
+	fn split_name_on_namestr_with_valid_name() {
+		let nm: &NameStr = "foo:bar".try_into().unwrap();
+		let (prefix, localname) = nm.split_name().unwrap();
+		assert_eq!(prefix.unwrap(), "foo");
+		assert_eq!(localname, "bar");
+	}
+
+	#[test]
+	fn split_name_on_namestr_with_prefixless_name() {
+		let nm: &NameStr = "bar".try_into().unwrap();
+		let (prefix, localname) = nm.split_name().unwrap();
+		assert_eq!(prefix, None);
+		assert_eq!(localname, "bar");
+	}
+
+	#[test]
+	fn split_name_on_namestr_rejects_localname_with_non_namestart_first_char() {
+		let nm: &NameStr = "foo:-bar".try_into().unwrap();
+		let result = nm.split_name();
+		assert!(matches!(
+			result.err().unwrap(),
+			XmlError::InvalidLocalName(_)
+		));
+	}
+
+	#[test]
+	fn split_name_on_name_with_valid_name() {
+		let nm: Name = "foo:bar".try_into().unwrap();
+		let (prefix, localname) = nm.split_name().unwrap();
+		assert_eq!(prefix.unwrap(), "foo");
+		assert_eq!(localname, "bar");
+	}
+
+	#[test]
+	fn split_name_on_name_with_prefixless_name() {
+		let nm: Name = "bar".try_into().unwrap();
+		let (prefix, localname) = nm.split_name().unwrap();
+		assert_eq!(prefix, None);
+		assert_eq!(localname, "bar");
+	}
+
+	#[test]
+	fn split_name_on_name_rejects_localname_with_non_namestart_first_char() {
 		let nm: Name = "foo:-bar".try_into().unwrap();
 		let result = nm.split_name();
 		assert!(matches!(
 			result.err().unwrap(),
-			NWFError::InvalidLocalName(_)
+			XmlError::InvalidLocalName(_)
 		));
 	}
 
@@ -777,3 +965,7 @@ mod tests {
 		let _: &CDataStr = "http://www.w3.org/XML/1998/namespace".try_into().unwrap();
 	}
 }
+
+/// Compatibility alias, use [`NcName`] directly instead.
+#[deprecated(since = "0.8.0", note = "type was renamed to NcName")]
+pub type NCName = NcName;
